#include <assert.h>
#include "ntHash.h"
#include "ntMalloc.h"

#define InitialHashSize  16
#define HashResizeRatio  3
#define HashResizeFactor 16

void ntHashNew(NtHash **pHcont)
{
  int i;
  *pHcont = ntMalloc(sizeof(NtHash));
  (*pHcont)->cHash  = InitialHashSize;
  (*pHcont)->cEntry = 0;
  (*pHcont)->aHash = ntMalloc(sizeof(NtList *) * InitialHashSize);
  for(i=0;i<InitialHashSize;i++)
    (*pHcont)->aHash[i] = NULL;
}

void ntHashFree(NtHash **pHcont)
{
  int i;
  NtList *pair;
  
  for(i=0;i<(*pHcont)->cHash<InitialHashSize;i++){
    while(ntListPop(&(*pHcont)->aHash[i],&pair)){
      ntStringFree((char **)&pair->p);
      ntStringFree((char **)&pair->next);
      ntListFree(pair);
    }
  }
  ntFreePtr(&((*pHcont)->aHash));
  ntFreePtr(pHcont);
}

void ntHashSet(NtHash *hcont, const char *name, const char *value)
{
  unsigned int h;
  NtList *p,*pair;
  char *n;
  
  if(hcont->cEntry++ > hcont->cHash * HashResizeRatio)
    ntHashResize(hcont);
  
  h = ntHashValue(hcont, name);
  for(p=hcont->aHash[h];p!=NULL;p=p->next){
    pair = (NtList *)p->p;
    n = (char *)pair->p;
    if(strcmp(name,n)==0) break;
  }

  if(p != NULL)
    ntStringSet((char **)&pair->next,value);
  else {
    pair = ntListCons(ntStringNew(name), (NtList *)ntStringNew(value));
    ntListPush(&hcont->aHash[h],pair);
  }
}

int ntHashFind(NtHash *hcont, const char *name,char **pValue)
{
  unsigned int h = ntHashValue(hcont, name);
  NtList *p,*pair;

  for(p=hcont->aHash[h];p!=NULL;p=p->next){
    pair = (NtList *)p->p;
    if(strcmp(name,(char *)pair->p)==0){
      *pValue = (char *)pair->next;
      return 1;
    }
  }
  return 0;
}

unsigned int ntHashValue(NtHash *hcont,const char *name)
{
  unsigned int hash = 0;
  const char *p;

  for(p=name;*p;p++)
    hash = hash * 1573 + *p * 14257;
  return hash % hcont->cHash;
}

void ntHashResize(NtHash *hcont)
{
  int cEntry = hcont->cEntry;
  int cHash  = hcont->cHash;
  int cHash2 = cHash * HashResizeFactor;
  NtList **aHash  = hcont->aHash;
  NtList **aHash2 = ntMalloc(sizeof(NtList *) * cHash2);
  NtList *pair;
  int i;
  static int bInResize = 0;

  assert(bInResize == 0);
  bInResize = 1;
  
  hcont->aHash = aHash2;
  hcont->cHash = cHash2;
  hcont->cEntry = 0;

  for(i=0;i<hcont->cHash;i++){
    while(ntListPop(&aHash[i],&pair)){
      char *n = (char *)pair->p;
      char *v = (char *)pair->next;
      ntHashSet(hcont, n, v);
      ntStringFree(&n);
      ntStringFree(&v);
      ntListFree(pair);
    }
  }
  ntFreePtr(&aHash);
  bInResize = 0;
  assert(cEntry == hcont->cEntry);
}

void ntHashDump(FILE *ofp,NtHash *hcont)
{
  int i;
  NtList *p,*pair;
  char   *n,*v;
  
  fprintf(ofp,"Hash(%p,size:%d,entry:%d)\n",hcont,hcont->cHash,hcont->cEntry);
  for(i=0;i<hcont->cHash;i++){
    fprintf(ofp," [%d] entry:%d",i,ntListLength(hcont->aHash[i]));
    for(p=hcont->aHash[i];p!=NULL;p++){
      pair = (NtList *)p->p;
      n = (char *)pair->p;
      v = (char *)pair->next;
      fprintf(ofp,"%s->%s ",n,v);
    }
    fprintf(ofp,"\n");
  }
}
