#include "ntLib.h"

static NtList *ntListCells = NULL;

#define NtListBlockSize 1024

NtList *ntListCons(car, cdr)
    void   *car;
    NtList *cdr;
{
  NtList *cell;
  
  if(ntListCells == NULL){
    NtList *tbl;
    int    size,i;
    
    size = NtListBlockSize * sizeof(NtList);
    tbl = ntMalloc(size);
    for(i=0; i < NtListBlockSize-1; i++)
      tbl[i].next = &tbl[i+1];
    tbl[i].next = NULL;
    ntListCells = &tbl[0];
  }
  cell = ntListCells;
  ntListCells = ntListCells->next;
  cell->p     = car;
  cell->next  = cdr;
  return cell;
}

void ntListFree(cell)
    NtList *cell;
{
  cell->next = ntListCells;
  ntListCells = cell;
}

NtList *ntListFreeAll(list)
    NtList *list;
{
  NtList *p0,*p;

  if(list != NULL){
    for(p0=NULL,p=list; p!=NULL; p0=p,p=p->next);
    assert(p0 != NULL);
    p0->next = ntListCells;
    ntListCells = list;
  }
  return NULL;
}

void ntListPtrFree(pList)
    NtList **pList;
{
  if(*pList != NULL)
    *pList = ntListFreeAll(*pList);
}

void ntListPush(pList, data)
    NtList **pList;
    void   *data;
{
  *pList = ntListCons(data, *pList);
}

int ntListPop(pList, pData0)
    NtList **pList;
    void   *pData0;
{
  NtList *list;
  void **pData = pData0;
  
  list = *pList;
  if(list == NULL) return 0;
  *pData = list->p;
  *pList = list->next;
  ntListFree(list);
  return 1;
}

void ntListAddLast(pList, x)
    NtList **pList;
    void   *x;
{
  NtList **pp;

  for(pp = pList; (*pp)!=NULL;  pp = &((*pp)->next));
  *pp = ntListCons(x,NULL);
}

int ntListLength(list)
    NtList *list;
{
  NtList *p;
  int len;

  len = 0;
  for(p=list; p; p=p->next)
    len++;
  return len;
}

int ntListDelete(pList, vp)
    NtList **pList;
    void *vp;
{
  NtList *p,*p0;

  for(p = *pList, p0 = NULL; p != NULL; p0=p, p=p->next){
    if(p->p == vp){
      if(p0 != NULL) p0->next = p->next;
      else *pList = p->next;
      ntListFree(p);
      return 1;
    }
  }
  return 0;
}

NtList *ntListReverse(list)
    NtList *list;
{
  NtList *p,*rList = NULL;

  for(p=list; p;p=p->next)
    rList = ntListCons(p->p, rList);
  return rList; 
}

void ntQueueClear(queue)
    NtQueue *queue;
{
  ntListPtrFree(&queue->head);
  queue->tail = NULL;
}

void ntQueueAdd(queue, vp)
    NtQueue *queue;
    void *vp;
{
  NtList *new;

  new = ntListCons(vp,NULL);
  if(queue->tail == NULL)
    queue->head = queue->tail = new;
  else {
    queue->tail->next = new;
    queue->tail = new;
  }
}

int ntQueuePop(queue, pData0)
    NtQueue *queue;
    void *pData0;
{
  void **pData = pData0;
  NtList *cell;
  
  if(queue->head == NULL){
    assert(queue->tail == NULL);
    return 0;
  }

  cell = queue->head;
  *pData = cell->p;
  if(cell->next == NULL){
    assert(queue->tail == cell);
    queue->head = queue->tail = NULL;
  }
  else queue->head = cell->next;
  ntListFree(cell);
  return 1;
}

int ntListNth(list, n, vp)
    NtList *list;
    int     n;
    void   *vp;
{
  void **pp = vp;
  NtList *p;
  int i;

  if(n < 0) p = NULL;
  else
    for(i=0, p=list; i < n; i++, p=p->next)
      if(p == NULL) break;

  if(p == NULL) {
    if(pp) *pp = NULL;
    return 0;
  }
  else {
    if(pp) *pp = p->p;
    return 1;
  }
}

NtList *ntListCopy(list)
    NtList *list;
{
  NtQueue q;
  NtList *p;
  
  q.head = q.tail = NULL;
  for(p=list; p;p=p->next)
    ntQueueAdd(&q, p->p);
  return q.head;
}

NtList *ntListGet(list, vp)
    NtList *list;
    void *vp;
{
  NtList *p;
  
  for(p=list; p!=NULL; p=p->next)
    if(p->p == vp) break;
  return p;
}
