#include "ntLib.h"
#include <stdlib.h>
#include <stdarg.h>

void *ntMalloc(size)
    int size;
{
  void *block;

  assert(size > 0);
  block = malloc(size);
  if(block == NULL){
    fprintf(stderr,"Memory Allocation Failed. size = %d bytes",size);
    exit(1);
  }
  memset(block, 0, size);
  return block;
}

void ntFreePtr(pBlock0)
    void *pBlock0;
{
  void **pBlock = pBlock0;
  if(*pBlock != NULL){
    free(*pBlock);
    *pBlock = NULL;
  }
}

void *ntCalloc(size)
    int size;
{
  void *block;

  block = ntMalloc(size);
  memset(block, 0, size);
  return block;
}

char *ntStringNew(str)
     const char *str;
{
  char *dup;
  int  len;

  if(str == NULL) return NULL;
  len = strlen(str);
  dup = ntMalloc(len+1);
  strcpy(dup,str);
  return dup;
}

void ntStringFree(pStr)
    char **pStr;
{
  if(*pStr != NULL)
    ntFreePtr(pStr);
}

void ntStringSet(pVar, str)
    char **pVar;
    const char *str;
{
  ntStringFree(pVar);
  *pVar = str ? ntStringNew(str) : NULL;
}

char *ntFmtString(char *fmt, ...)
{
  va_list ap;
  char buf[256];

  va_start(ap, fmt);
  vsnprintf(buf,sizeof(buf),fmt,ap);
  return ntStringNew(buf);
}
